'Goodies (11/18/2001 8:34:50 am)'.!


ListView subclass: #PluggableYAListView
	instanceVariableNames: 'items getListSelector getSelectionSelector setSelectionSelector getMenuSelector keystrokeActionSelector '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pluggable-Support'!
!PluggableYAListView commentStamp: '<historical>' prior: 0!
from Squeak 1.31!


!PluggableYAListView methodsFor: 'initialization'!
list: arrayOfStrings
	"Set the receivers items to be the given list of strings."
	"Note: the instance variable 'itemsList' holds the original list.
	 The instance variable 'items' is a paragraph constructed from
	 this list."

	| s |
	items _ arrayOfStrings.
	isEmpty _ arrayOfStrings isEmpty.

	s _ WriteStream on: Array new.
	"add top and bottom delimiters"
	s nextPut: topDelimiter.
	arrayOfStrings do: [:item |
		item == nil ifFalse: [s nextPut: item].
	].
	s nextPut: bottomDelimiter.
	list _ ListParagraph withArray: s contents.

	selection _ self getCurrentSelectionIndex.
	self positionList.! !

!PluggableYAListView methodsFor: 'initialization'!
on: anObject list: getListSel selected: getSelectionSel changeSelected: setSelectionSel menu: getMenuSel keystroke: keyActionSel

	self model: anObject.
	getListSelector _ getListSel.
	getSelectionSelector _ getSelectionSel.
	setSelectionSelector _ setSelectionSel.
	getMenuSelector _ getMenuSel.
	keystrokeActionSelector _ keyActionSel.
	self borderWidth: 1.
	self list: self getList.! !


!PluggableYAListView methodsFor: 'model access'!
changeModelSelection: anInteger
	"Change the model's selected item to be the one at the given index."

	| item |
	setSelectionSelector ~~ nil ifTrue: [
		item _ (anInteger = 0 ifTrue: [nil] ifFalse: [items at: anInteger]).
		model perform: setSelectionSelector with: item.
		getSelectionSelector == nil ifFalse: [model perform: getSelectionSelector].
	].! !

!PluggableYAListView methodsFor: 'model access'!
getCurrentSelectionIndex
	"Answer the index of the current selection."

	| item |
	getSelectionSelector == nil ifTrue: [^ 0].
	item _ model perform: getSelectionSelector.
	^ items findFirst: [ :x | x = item]
! !

!PluggableYAListView methodsFor: 'model access'!
getList 
	"Answer the list to be displayed."

	| lst |
	getListSelector == nil ifTrue: [^ #()].
	lst _ model perform: getListSelector.
	lst == nil ifTrue: [^ #()].
	^ lst! !

!PluggableYAListView methodsFor: 'model access'!
getMenu
	"Answer the menu for this list view."

	getMenuSelector == nil ifTrue: [^ nil].
	^ model perform: getMenuSelector! !

!PluggableYAListView methodsFor: 'model access'!
handleKeystroke: aCharacter
	"Answer the menu for this list view."

	keystrokeActionSelector == nil ifTrue: [^ nil].
	model perform: keystrokeActionSelector with: aCharacter.! !


!PluggableYAListView methodsFor: 'controller access'!
defaultControllerClass 

	^ PluggableYAListController! !


!PluggableYAListView methodsFor: 'updating'!
update: aSymbol 
	"Refer to the comment in View|update:."

	| oldIndex newIndex |
	aSymbol == getListSelector ifTrue: [
		oldIndex _ self getCurrentSelectionIndex.
		self list: self getList.
		newIndex _ self getCurrentSelectionIndex.
		(oldIndex > 0 and: [newIndex = 0]) ifTrue: [
			"new list did not include the old selection; deselecting"
			self changeModelSelection: newIndex].
		self displayView.
		self displaySelectionBox.
		^self].
	aSymbol == getSelectionSelector ifTrue: [
		self moveSelectionBox: self getCurrentSelectionIndex.
		^self].
! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

PluggableYAListView class
	instanceVariableNames: ''!

!PluggableYAListView class methodsFor: 'as yet unclassified'!
aboutPluggability
	"A pluggable list view gets its content from the model. This allows the same kind of view can be used in different situations, thus avoiding a proliferation of gratuitous view and controller classes. Selector usage is:

		getListSel		fetch the list of items (strings) to be displayed
		getSelectionSel	get the currently selected item
		setSelectionSel	set the currently selected item (takes an argument)
		getMenuSel		get the pane-specific (or 'yellow-button') menu
		keyActionSel	process keystrokes typed to this view (takes an argument)

	Any of the above selectors can be nil, meaning that the model does not supply behavior for the given action, and the default, do-nothing behavior should be used. However, if getListSel is nil, the default behavior just provides an empty list, which makes for a rather dull list view!! (Such behavior can actually be useful during debugging.)

	The model informs a pluggable view of changes by sending #changed: to itself with getListSel or getSelectionSel as a parameter. The view informs the model of selection changes by sending setSelectionSel to it with the newly selected item as a parameter, and invokes menu and keyboard actions on the model via getMenuSel and keyActionSel.

	Pluggability allows a single model object to have pluggable list views on multiple aspects of itself. For example, an object representing one personal music library might be organized as a three-level hierarchy: the types of music, the titles within a given type, and the songs on a given title. Pluggability allows one to easily build a multipane browser for this object with separate list views for the music type, title, and song."! !

!PluggableYAListView class methodsFor: 'as yet unclassified'!
on: anObject list: getListSel selected: getSelectionSel changeSelected: setSelectionSel
	"Create a 'pluggable' list view on the given model parameterized by the given message selectors. See aboutPluggability comment."

	^ self new
		on: anObject
		list: getListSel
		selected: getSelectionSel
		changeSelected: setSelectionSel
		menu: nil
		keystroke: nil
! !

!PluggableYAListView class methodsFor: 'as yet unclassified'!
on: anObject list: getListSel selected: getSelectionSel changeSelected: setSelectionSel menu: getMenuSel
	"Create a 'pluggable' list view on the given model parameterized by the given message selectors. See aboutPluggability comment."

	^ self new
		on: anObject
		list: getListSel
		selected: getSelectionSel
		changeSelected: setSelectionSel
		menu: getMenuSel
		keystroke: nil
! !

!PluggableYAListView class methodsFor: 'as yet unclassified'!
on: anObject list: getListSel selected: getSelectionSel changeSelected: setSelectionSel menu: getMenuSel keystroke: keyActionSel
	"Create a 'pluggable' list view on the given model parameterized by the given message selectors. See aboutPluggability comment."

	^ self new
		on: anObject
		list: getListSel
		selected: getSelectionSel
		changeSelected: setSelectionSel
		menu: getMenuSel
		keystroke: keyActionSel
! !
