'Goodies (11/18/2001 8:34:54 am)'.!


Object subclass: #LispScanner
	instanceVariableNames: 'source mark token tokenType typeTable failBlock compiler '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Goodies-Lisp'!

!LispScanner methodsFor: 'initialize-release' stamp: 'nishis 4/25/98 04:05'!
initScanner
	| label string |
	typeTable := LispScannerTable scannerTable.
	failBlock := 
			[:errorMessage | 
			label := errorMessage , ' near ' , (token printString contractTo: 10).
			string := source upToEnd.
			string isEmpty
				ifTrue: [string := '--> end of file']
				ifFalse: [string := '--> ' , (string contractTo: 30)].
			self error: 'scan error.' with: label, '\' withCRs , string].
	Compiler isNil
		ifTrue: ["when I am a run time image"
			compiler := LispSmallCompiler new]
		ifFalse: [compiler := Compiler new]! !

!LispScanner methodsFor: 'initialize-release'!
on: inputStream 
	source := inputStream.
	mark := source position! !


!LispScanner methodsFor: 'scanning'!
multiChar: type 
	self perform: type! !

!LispScanner methodsFor: 'scanning'!
nextToken
	| char |
	mark := source position.
	char := self peekChar.
	tokenType := typeTable at: char asInteger.
	[tokenType == #xDelimiter]
		whileTrue: 
			[self nextChar.
			char := self peekChar.
			tokenType := typeTable at: char asInteger].
	(tokenType at: 1)
		= $x
		ifTrue: [self multiChar: tokenType]
		ifFalse: [self singleChar: tokenType].
	^token! !

!LispScanner methodsFor: 'scanning'!
singleChar: type 
	self nextChar.
	token := type! !

!LispScanner methodsFor: 'scanning'!
unNextToken
	source position: mark! !


!LispScanner methodsFor: 'x'!
xBinary
	^failBlock value: 'Syntax error ' , source peek printString! !

!LispScanner methodsFor: 'x'!
xBrace
	tokenType := #object.
	token := self objectFrom: source! !

!LispScanner methodsFor: 'x'!
xComment
	| char |
	[(char := self nextChar) = Character cr]
		whileFalse: [char == nil ifTrue: [^self nextToken]].
	^self nextToken! !

!LispScanner methodsFor: 'x'!
xDigit
	tokenType := #number.
	token := self numberFrom: source! !

!LispScanner methodsFor: 'x'!
xDoubleQuote
	tokenType := #string.
	token := self stringFrom: source! !

!LispScanner methodsFor: 'x'!
xSign
	| char sign |
	sign := self nextChar.
	char := self peekChar.
	char isDigit
		ifTrue: 
			[tokenType := #number.
			token := self numberFrom: source.
			sign == $- ifTrue: [token := token negated]]
		ifFalse: 
			[self unNextChar.
			tokenType := #symbol.
			token := self symbolFrom: source]! !

!LispScanner methodsFor: 'x'!
xSymbol
	tokenType := #symbol.
	token := self symbolFrom: source! !


!LispScanner methodsFor: 'reading'!
numberFrom: aStream 
	^Number readFrom: aStream! !

!LispScanner methodsFor: 'reading'!
objectFrom: aStream 
	| buffer char |
	buffer := WriteStream on: (String new: 20).
	char := aStream next.
	
	[char := aStream next.
	char = $}]
		whileFalse: 
			[char == nil ifTrue: [^failBlock value: 'Syntax error unmatched ${'].
			buffer nextPut: char].
	^compiler
		evaluate: buffer contents
		in: nil
		to: nil
		notifying: nil
		ifFail: []! !

!LispScanner methodsFor: 'reading'!
stringFrom: aStream 
	| buffer char string |
	buffer := WriteStream on: (String new: 20).
	char := aStream next.
	char = $" ifTrue: [
		[char := aStream peek.
		char ~~ nil]
			whileTrue: 
				[char = $"
					ifTrue: 
						[aStream next.
						char := aStream peek.
						char = $" ifFalse: [^String fromString: buffer contents]].
				buffer nextPut: aStream next]].
	string := aStream upToEnd.
	string size > 100 ifTrue: [string := string copyFrom: 1 to: 100].
	^failBlock value: 'Syntax error unmatched $'''! !

!LispScanner methodsFor: 'reading'!
symbolFrom: aStream 
	| buffer char type |
	buffer := WriteStream on: (String new: 20).
	char := aStream peek.
	[char notNil and: [(type := typeTable at: char asInteger) == #xSymbol or: [type == #xDigit or: [type == #xSign]]]]
		whileTrue: 
			[buffer nextPut: aStream next.
			char := aStream peek].
	buffer contents = 'nil' ifTrue: [^LispNil null].
	^Symbol intern: buffer contents! !


!LispScanner methodsFor: 'error handling' stamp: 'nishis 6/28/1998 06:02'!
error: labelString with: contentsString
	| handler |
	(handler _ Processor activeProcess errorHandler) notNil
		ifTrue:
			[handler value: labelString value: self]
		ifFalse: 
			[Debugger
				openContext: thisContext
				label: labelString
				contents: contentsString]! !


!LispScanner methodsFor: 'private'!
nextChar
	| char |
	char := source next.
	char = Character linefeed ifTrue: [char := Character cr].
	char = Character cr
		ifTrue: 
			[source peek = Character linefeed ifTrue: [source next].
			char := Character cr].
	char isNil ifTrue: [char := $!!].
	^char! !

!LispScanner methodsFor: 'private'!
peekChar
	| char |
	char := source peek.
	char = Character linefeed ifTrue: [char := Character cr].
	char = Character cr ifTrue: [char := Character cr].
	char isNil ifTrue: [char := $!!].
	^char! !

!LispScanner methodsFor: 'private'!
unNextChar
	source skip: -1! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

LispScanner class
	instanceVariableNames: ''!

!LispScanner class methodsFor: 'copyright'!
copyright
	^'Copyright (C) 1995-1998 AOKI Atsushi, All Rights Reserved.'! !

!LispScanner class methodsFor: 'copyright'!
system
	^'Goodies'! !

!LispScanner class methodsFor: 'copyright'!
version
	^'003'! !


!LispScanner class methodsFor: 'instance creation'!
new
	^(super new) initScanner; yourself! !
