'Goodies (11/18/2001 8:35:14 am)'.!


LispList subclass: #LispCons
	instanceVariableNames: 'head tail '
	classVariableNames: 'HorizontalLevel VerticalLevel '
	poolDictionaries: ''
	category: 'Goodies-Lisp'!

!LispCons methodsFor: 'accessing'!
at: indexInteger
	| count |
	count := 1.
	self
		mapcdr: 
			[:cdr | 
			indexInteger = count ifTrue: [^cdr head].
			count := count + 1].
	^self subscriptOutOfBoundsError: indexInteger! !

!LispCons methodsFor: 'accessing'!
at: indexInteger put: anObject 
	| count |
	count := 1.
	self
		mapcdr: 
			[:cdr | 
			indexInteger = count ifTrue: [^cdr head: anObject].
			count := count + 1].
	^self subscriptOutOfBoundsError: indexInteger! !

!LispCons methodsFor: 'accessing'!
head
	^head! !

!LispCons methodsFor: 'accessing'!
head: anObject 
	^head := anObject! !

!LispCons methodsFor: 'accessing'!
head: headObject tail: tailObject 
	self head: headObject.
	self tail: tailObject! !

!LispCons methodsFor: 'accessing'!
tail
	^tail! !

!LispCons methodsFor: 'accessing'!
tail: anObject 
	^tail := anObject! !


!LispCons methodsFor: 'testing'!
= anObject 
	(anObject isKindOf: self class)
		ifFalse: [^false].
	self head = anObject head ifTrue: [^self tail = anObject tail].
	^false! !


!LispCons methodsFor: 'adding'!
add: newObject 
	^self nconc: (self class head: newObject tail: self class null)! !


!LispCons methodsFor: 'enumerating'!
collect: aBlock 
	| list result |
	list := self.
	result := self class null.
	[list isKindOf: self class]
		whileTrue: 
			[result := self class head: (aBlock value: list head)
						tail: result.
			list := list tail].
	^result reverse! !

!LispCons methodsFor: 'enumerating'!
do: aBlock 
	| list |
	list := self.
	[list isKindOf: self class]
		whileTrue: 
			[aBlock value: list head.
			list := list tail]! !


!LispCons methodsFor: 'functions'!
append: list 
	(tail isKindOf: self class)
		ifFalse: [^self class head: head tail: list].
	^self class head: head tail: (tail append: list)! !

!LispCons methodsFor: 'functions'!
last
	| list |
	list := self class head: nil tail: self.
	self do: [:each | list := list tail].
	^list! !

!LispCons methodsFor: 'functions'!
length
	| count |
	count := 0.
	self do: [:each | count := count + 1].
	^count! !

!LispCons methodsFor: 'functions'!
mapcdr: aBlock 
	| list |
	list := self.
	[list isKindOf: self class]
		whileTrue: 
			[aBlock value: list.
			list := list tail]! !

!LispCons methodsFor: 'functions'!
member: anObject 
	| list |
	list := self.
	self do: 
		[:each | 
		each = anObject ifTrue: [^list].
		list := list tail].
	^self species null! !

!LispCons methodsFor: 'functions'!
memq: anObject 
	| list |
	list := self.
	self do: 
		[:each | 
		each == anObject ifTrue: [^list].
		list := list tail].
	^self species null! !

!LispCons methodsFor: 'functions'!
nconc: list 
	self last rplacd: list! !

!LispCons methodsFor: 'functions'!
nth: nth 
	| count list |
	nth <= 0 ifTrue: [^self species null].
	count := 1.
	list := self.
	list do: 
		[:each | 
		count >= nth ifTrue: [^each].
		count := count + 1].
	^self species null! !

!LispCons methodsFor: 'functions'!
reverse
	| list |
	list := self class null.
	self do: [:each | list := self class head: each tail: list].
	^list! !

!LispCons methodsFor: 'functions'!
rplaca: anObject 
	self head: anObject! !

!LispCons methodsFor: 'functions'!
rplacd: anObject 
	self tail: anObject! !


!LispCons methodsFor: 'printing'!
printOn: aStream 
	self printOn: aStream level: 0! !

!LispCons methodsFor: 'printing'!
printOn: aStream level: level 
	| verticalLevel |
	verticalLevel := self class verticalLevel.
	(verticalLevel ~= 0 and: [level >= verticalLevel])
		ifTrue: 
			[aStream nextPutAll: '( ... )'.
			^self].
	self null ifTrue: [^super printOn: aStream].
	aStream nextPutAll: '('.
	(head isKindOf: self class)
		ifTrue: [head printOn: aStream level: level + 1]
		ifFalse: [self printOn: aStream object: head].
	(tail isKindOf: LispList)
		ifTrue: [self
				printOn: aStream
				tail: tail
				level: level]
		ifFalse: 
			[aStream nextPutAll: ' . '.
			self printOn: aStream object: tail.
			^aStream nextPutAll: ')']! !

!LispCons methodsFor: 'printing'!
printOn: aStream object: anObject 
	(anObject isKindOf: Symbol)
		ifTrue: [^aStream nextPutAll: anObject asString].
	(anObject isKindOf: String)
		ifTrue: 
			[aStream nextPutAll: '"'.
			anObject
				do: 
					[:char | 
					char = $" ifTrue: [aStream nextPut: $"].
					aStream nextPut: char].
			^aStream nextPutAll: '"'].
	(anObject isKindOf: Number)
		ifTrue: [^anObject storeOn: aStream].
	(anObject isMemberOf: LispNil)
		ifTrue: [^aStream nextPutAll: 'nil'].
	aStream nextPutAll: '{'.
	aStream nextPutAll: (anObject printString contractTo: 80).
	aStream nextPutAll: '}'! !

!LispCons methodsFor: 'printing'!
printOn: aStream tail: cdr level: level 
	| tailPart count horizontalLevel |
	cdr null ifTrue: [^aStream nextPutAll: ')'].
	tailPart := cdr.
	count := 1.
	horizontalLevel := self class horizontalLevel.
	tailPart do: 
		[:each | 
		(horizontalLevel ~= 0 and: [count >= horizontalLevel])
			ifTrue: 
				[aStream nextPutAll: ' ... )'.
				^self].
		aStream nextPutAll: ' '.
		(each isKindOf: self class)
			ifTrue: [tailPart head printOn: aStream level: level + 1]
			ifFalse: [self printOn: aStream object: each].
		tailPart := tailPart tail.
		count := count + 1].
	(tailPart isKindOf: LispList)
		ifTrue: [aStream nextPutAll: ')']
		ifFalse: 
			[aStream nextPutAll: ' . '.
			self printOn: aStream object: tailPart.
			aStream nextPutAll: ')']! !


!LispCons methodsFor: 'pretty printing'!
ppOn: aStream 
	self
		ppOn: aStream
		list: self
		position: 0.
	aStream cr! !

!LispCons methodsFor: 'pretty printing'!
ppOn: aStream list: list position: position 
	(list isKindOf: self class)
		ifFalse: [^self ppOn: aStream object: list].
	(list head isKindOf: self class) not
		ifTrue: 
			[aStream nextPutAll: '('.
			self ppOn: aStream object: list head.
			(list tail isKindOf: LispList)
				ifTrue: [self
						ppOn: aStream
						tail: list tail
						position: position + 1]
				ifFalse: 
					[aStream nextPutAll: ' . '.
					self ppOn: aStream object: list tail].
			aStream nextPutAll: ')']
		ifFalse: 
			[aStream nextPutAll: '('.
			self
				ppOn: aStream
				list: list head
				position: position + 1.
			(list tail isKindOf: self class)
				ifTrue: [(list tail head isKindOf: self class)
						ifTrue: 
							[aStream cr.
							self ppOn: aStream spaceAndTab: position.
							self
								ppOn: aStream
								tail: list tail
								position: position]
						ifFalse: 
							[self ppOn: aStream space: 1.
							self
								ppOn: aStream
								tail: list tail
								position: position + 1]]
				ifFalse: [(list tail isKindOf: LispList)
						ifFalse: 
							[aStream nextPutAll: ' . '.
							self ppOn: aStream object: list tail]].
			aStream nextPutAll: ')']! !

!LispCons methodsFor: 'pretty printing'!
ppOn: aStream object: anObject 
	(anObject isKindOf: Symbol)
		ifTrue: [^aStream nextPutAll: anObject asString].
	(anObject isKindOf: String)
		ifTrue: 
			[aStream nextPutAll: '"'.
			anObject
				do: 
					[:char | 
					char = $" ifTrue: [aStream nextPut: $"].
					aStream nextPut: char].
			^aStream nextPutAll: '"'].
	(anObject isKindOf: Number)
		ifTrue: [^anObject storeOn: aStream].
	(anObject isMemberOf: LispNil)
		ifTrue: [^aStream nextPutAll: 'nil'].
	aStream nextPutAll: '{'.
	aStream nextPutAll: (anObject printString contractTo: 80).
	aStream nextPutAll: '}'! !

!LispCons methodsFor: 'pretty printing'!
ppOn: aStream space: anInteger 
	anInteger timesRepeat: [aStream nextPut: Character space]! !

!LispCons methodsFor: 'pretty printing'!
ppOn: aStream spaceAndTab: anInteger 
	| tabs spaces |
	tabs := anInteger // self tabStop.
	spaces := anInteger \\ self tabStop.
	tabs * (self tabStop // 4) timesRepeat: [aStream tab].
	spaces timesRepeat: [aStream space]! !

!LispCons methodsFor: 'pretty printing'!
ppOn: aStream tail: list position: position 
	list null ifTrue: [^self].
	(list tail isKindOf: LispList)
		ifTrue: [list tail null
				ifTrue: 
					[self ppOn: aStream space: 1.
					self
						ppOn: aStream
						list: list head
						position: position + 1]
				ifFalse: 
					[self ppOn: aStream space: 1.
					self
						ppOn: aStream
						list: list head
						position: position + 1.
					aStream cr.
					self ppOn: aStream spaceAndTab: position.
					self
						ppOn: aStream
						tail: list tail
						position: position]]
		ifFalse: 
			[self ppOn: aStream space: 1.
			self
				ppOn: aStream
				list: list head
				position: position + 1.
			aStream nextPutAll: ' . '.
			self ppOn: aStream object: list tail]! !

!LispCons methodsFor: 'pretty printing'!
ppString
	| stream |
	stream := WriteStream on: (String new: 20).
	self ppOn: stream.
	^stream contents! !


!LispCons methodsFor: 'saving'!
saveOn: aStream 
	self
		saveOn: aStream
		list: self
		position: 0.
	aStream cr! !

!LispCons methodsFor: 'saving'!
saveOn: aStream list: list position: position 
	| location length |
	(list isKindOf: self class)
		ifFalse: [^self saveOn: aStream object: list].
	(list head isKindOf: self class) not
		ifTrue: 
			[aStream nextPutAll: '('.
			location := aStream position.
			self saveOn: aStream object: list head.
			(list tail isKindOf: LispList)
				ifTrue: 
					[length := aStream position - location min: 40.
					length := 0.
					self
						saveOn: aStream
						tail: list tail
						position: position + 1 + length]
				ifFalse: 
					[aStream nextPutAll: ' . '.
					self saveOn: aStream object: list tail].
			aStream nextPutAll: ')']
		ifFalse: 
			[aStream nextPutAll: '('.
			self
				saveOn: aStream
				list: list head
				position: position + 1.
			(list tail isKindOf: self class)
				ifTrue: [(list tail head isKindOf: self class)
						ifTrue: 
							[aStream cr.
							self saveOn: aStream spaceAndTab: position.
							self
								saveOn: aStream
								tail: list tail
								position: position]
						ifFalse: 
							[self saveOn: aStream space: 1.
							self
								saveOn: aStream
								tail: list tail
								position: position + 1]]
				ifFalse: [(list tail isKindOf: LispList)
						ifFalse: 
							[aStream nextPutAll: ' . '.
							self saveOn: aStream object: list tail]].
			aStream nextPutAll: ')']! !

!LispCons methodsFor: 'saving'!
saveOn: aStream object: anObject 
	| string |
	(anObject isKindOf: Symbol)
		ifTrue: [^aStream nextPutAll: anObject asString].
	(anObject isKindOf: String)
		ifTrue: 
			[aStream nextPutAll: '"'.
			anObject
				do: 
					[:char | 
					char = $" ifTrue: [aStream nextPut: $"].
					aStream nextPut: char].
			^aStream nextPutAll: '"'].
	(anObject isKindOf: Integer)
		ifTrue: [^anObject storeOn: aStream].
	(anObject isKindOf: Float)
		ifTrue: [^anObject storeOn: aStream].
	"(anObject isKindOf: Double)
		ifTrue: [^anObject storeOn: aStream]."
	(anObject isMemberOf: LispNil)
		ifTrue: [^aStream nextPutAll: 'nil'].
	aStream nextPutAll: '{'.
	((anObject isKindOf: Point)
		or: [anObject isKindOf: Rectangle])
		ifTrue: [string := anObject printString]
		ifFalse: [string := anObject storeString].
	aStream nextPutAll: string.
	aStream nextPutAll: '}'! !

!LispCons methodsFor: 'saving'!
saveOn: aStream space: anInteger 
	anInteger timesRepeat: [aStream nextPut: Character space]! !

!LispCons methodsFor: 'saving'!
saveOn: aStream spaceAndTab: anInteger 
	| tabs spaces |
	tabs := anInteger // self tabStop.
	spaces := anInteger \\ self tabStop.
	tabs timesRepeat: [aStream tab].
	spaces timesRepeat: [aStream space]! !

!LispCons methodsFor: 'saving'!
saveOn: aStream tail: list position: position 
	list null ifTrue: [^self].
	(list tail isKindOf: LispList)
		ifTrue: [list tail null
				ifTrue: 
					[self saveOn: aStream space: 1.
					self
						saveOn: aStream
						list: list head
						position: position + 1]
				ifFalse: 
					[self saveOn: aStream space: 1.
					self
						saveOn: aStream
						list: list head
						position: position + 1.
					aStream cr.
					self saveOn: aStream spaceAndTab: position.
					self
						saveOn: aStream
						tail: list tail
						position: position]]
		ifFalse: 
			[self saveOn: aStream space: 1.
			self
				saveOn: aStream
				list: list head
				position: position + 1.
			aStream nextPutAll: ' . '.
			self saveOn: aStream object: list tail]! !

!LispCons methodsFor: 'saving'!
saveString
	| stream |
	stream := WriteStream on: (String new: 20).
	self saveOn: stream.
	^stream contents! !


!LispCons methodsFor: 'private'!
tabStop
	^8! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

LispCons class
	instanceVariableNames: ''!

!LispCons class methodsFor: 'copyright'!
copyright
	^'Copyright (C) 1995-1998 AOKI Atsushi, All Rights Reserved.'! !

!LispCons class methodsFor: 'copyright'!
system
	^'Goodies'! !

!LispCons class methodsFor: 'copyright'!
version
	^'003'! !


!LispCons class methodsFor: 'class initialization'!
initialize
	"LispCons initialize."

	HorizontalLevel := VerticalLevel := nil! !


!LispCons class methodsFor: 'instance creation'!
cell
	^super new head: self null tail: self null! !

!LispCons class methodsFor: 'instance creation'!
head: headObject 
	^super new head: headObject tail: self null! !

!LispCons class methodsFor: 'instance creation'!
head: headObject tail: tailObject 
	^super new head: headObject tail: tailObject! !

!LispCons class methodsFor: 'instance creation'!
list: anArray 
	| size list |
	size := anArray size.
	list := self null.
	size
		to: 1
		by: -1
		do: [:i | list := self head: (anArray at: i)
						tail: list].
	^list! !

!LispCons class methodsFor: 'instance creation'!
loadFrom: aStream 
	"by nishis, 1998/04/19 07:51"

	| list |
	list := LispParser parse: aStream.
	^list! !

!LispCons class methodsFor: 'instance creation'!
new
	^self cell! !

!LispCons class methodsFor: 'instance creation'!
null
	^LispNil null! !


!LispCons class methodsFor: 'level accessing'!
horizontalLevel
	HorizontalLevel isNil ifTrue: [HorizontalLevel := 50].
	^HorizontalLevel! !

!LispCons class methodsFor: 'level accessing'!
horizontalLevel: anInteger 
	HorizontalLevel := anInteger! !

!LispCons class methodsFor: 'level accessing'!
verticalLevel
	VerticalLevel isNil ifTrue: [VerticalLevel := 10].
	^VerticalLevel! !

!LispCons class methodsFor: 'level accessing'!
verticalLevel: anInteger 
	VerticalLevel := anInteger! !


!LispCons class methodsFor: 'utilities'!
classHierarchy: aClass 
	"LispCons classHierarchy: Number."

	| theClass list |
	aClass isMeta
		ifTrue: [theClass := aClass soleInstance]
		ifFalse: [theClass := aClass].
	list := self subclassHierarchy: theClass.
	(theClass allSuperclasses select: [:each | each isMeta not])
		do: [:each | list := self head: each name tail: (self head: list tail: self null)].
	^list! !

!LispCons class methodsFor: 'utilities'!
subclassHierarchy: aClass 
	"LispCons subclassHierarchy: Number."

	| theClass list collection sub |
	aClass isMeta
		ifTrue: [theClass := aClass soleInstance]
		ifFalse: [theClass := aClass].
	list := self null.
	theClass subclasses isEmpty
		ifFalse: 
			[collection := SortedCollection sortBlock: [:x :y | x name > y name].
			collection addAll: (theClass subclasses select: [:each | each isMeta not]).
			collection
				do: 
					[:each | 
					sub := self subclassHierarchy: each.
					list := self head: sub tail: list]].
	^self head: theClass name tail: list! !

!LispCons class methodsFor: 'utilities'!
superclassHierarchy: aClass 
	"LispCons superclassHierarchy: Number."

	| theClass list |
	aClass isMeta
		ifTrue: [theClass := aClass soleInstance]
		ifFalse: [theClass := aClass].
	list := self head: theClass name tail: self null.
	(theClass allSuperclasses select: [:each | each isMeta not])
		do: [:each | list := self head: each name tail: (self head: list tail: self null)].
	^list! !


!LispCons class methodsFor: 'examples'!
example1
	"LispCons example1."

	| list |
	list := LispCons list: #(1 2 3 4 5 6 7 8 9 10 ).
	Transcript cr; show: list printString.
	^list! !

!LispCons class methodsFor: 'examples'!
example2
	"LispCons example2."

	| null list |
	null := LispCons null.
	list := LispCons list: #(1 2 ).
	list := LispCons head: list tail: null.
	list := LispCons head: list tail: null.
	Transcript cr; show: list printString.
	^list! !

!LispCons class methodsFor: 'examples'!
example3
	"LispCons example3."

	| x y z |
	x := LispCons list: #(1 2 3 ).
	y := LispCons list: #(4 5 6 ).
	z := LispCons list: #(1 2 3 4 5 6 ).
	Transcript cr; show: '(setq x ''(1 2 3)) => ' , x printString.
	Transcript cr; show: '(setq y ''(4 5 6)) => ' , y printString.
	Transcript cr; show: '(setq z ''(1 2 3 4 5 6)) => ' , z printString.
	Transcript cr; show: '(append x y) => ' , (x append: y) printString.
	Transcript cr; show: '(length z) => ' , z length printString.
	Transcript cr; show: '(member 3 z) => ' , (z member: 3) printString.
	Transcript cr; show: '(nth 4 z) => ' , (z nth: 4) printString.
	^z! !

!LispCons class methodsFor: 'examples'!
example4
	"LispCons example4."

	| list |
	list := LispCons list: #(1 2 ).
	list := LispCons head: list tail: (LispCons list: #(3 4 )).
	list := LispCons head: list tail: (LispCons list: #(5 6 )).
	Transcript cr; show: list saveString.
	^list! !

!LispCons class methodsFor: 'examples'!
example5
	"LispCons example5."

	| list |
	list := LispCons loadFrom: '
			(PetriNet Aoki
				(Place p1 p2 p3 p4 p5)
				(Transition t1 t2 t3 t4 t5)
				(InputFunction
					(t1 p1 p2 p3 p4 p5)
					(t2 . p4)
					(t3 . p5))
				(OutputFunction
					(t1 p1 p2 p3 p4 p5)
					(t2 . p4)
					(t3 . p5))
				(Marking {#(1 2 3 4 5)})))'.
	Transcript cr; show: list saveString.
	^list! !

!LispCons class methodsFor: 'examples'!
example6
	"LispCons example6."

	| list |
	list := LispCons loadFrom: '(aaa bbb ccc)'.
	^list! !

!LispCons class methodsFor: 'examples'!
example7
	"LispCons example7."

	| list |
	list := LispCons loadFrom: ' `(`(1 2 `3) . `4 ) '.
	^list! !

!LispCons class methodsFor: 'examples'!
example8
	"LispCons example8."

	| list |
	list := LispCons loadFrom: ' ({100@100} {200@200} {Form fromUser}) '.
	^list! !

!LispCons class methodsFor: 'examples'!
example9
	"LispCons example9."

	LispCons null.
	LispCons cell.
	LispCons new: 5.
	LispCons head: 10 tail: (LispCons head: 20 tail: LispCons null).
	LispCons list: #(10 20 ).
	LispCons loadFrom: ' (10 {100@200} {Form fromUser}) '.
	(LispCons list: #(10 20 )) head.
	(LispCons list: #(10 20 )) tail.
	(LispCons list: #(10 20 ))
		append: (LispCons list: #(30 40 )).
	(LispCons list: #(10 20 )) last.
	(LispCons list: #(10 20 )) length.
	(LispCons list: #(10 20 ))
		nth: 2.
	(LispCons list: #(10 20 ))
		do: [:each | Transcript cr; show: each printString].
	(LispCons list: (0 to: 100)) printString.
	(LispCons list: (0 to: 100)) ppString.
	(LispCons list: (0 to: 100)) saveString! !


LispCons initialize!
