'Goodies (11/18/2001 8:35:16 am)'.!


LispScanner subclass: #LispParser
	instanceVariableNames: 'prevMark prevToken prevTokenType '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Goodies-Lisp'!

!LispParser methodsFor: 'parsing' stamp: 'nishis 4/25/98 04:05'!
parse: sourceStream 
	| label string |
	^self parse: sourceStream
		ifFail: 
			[:errorMessage | 
			label := errorMessage , ' near ' , (token printString contractTo: 20).
			string := source upToEnd.
			string isEmpty
				ifTrue: [string := '--> end of file']
				ifFalse: [string := '--> ' , (string contractTo: 30)].
			self error: 'parse error.' with: label, '\' withCRs , string.
			^LispNil null]! !

!LispParser methodsFor: 'parsing'!
parse: sourceStream ifFail: aBlock 
	| result |
	self init: sourceStream ifFail: aBlock.
	result := self scan.
	^result! !


!LispParser methodsFor: 'scanning'!
nextToken
	prevMark := mark.
	prevToken := token.
	prevTokenType := tokenType.
	^super nextToken! !

!LispParser methodsFor: 'scanning'!
unNextToken
	super unNextToken.
	mark := prevMark.
	token := prevToken.
	tokenType := prevTokenType! !


!LispParser methodsFor: 'scan and parse'!
scan
	source atEnd ifTrue: [^LispCons null].
	^self scanList! !

!LispParser methodsFor: 'scan and parse'!
scanList
	| expression |
	self nextToken.
	tokenType == #eof ifTrue: [^LispCons null].
	tokenType == #number ifTrue: [^token].
	tokenType == #string ifTrue: [^token].
	tokenType == #object ifTrue: [^token].
	tokenType == #symbol ifTrue: [^token].
	tokenType == #quote
		ifTrue: 
			[expression := LispCons head: self scanList tail: LispCons null.
			^LispCons head: #quote tail: expression].
	tokenType == #leftParenthesis ifTrue: [^self scanListAux].
	^failBlock value: 'Syntax error'! !

!LispParser methodsFor: 'scan and parse'!
scanListAux
	| cdr |
	self nextToken.
	tokenType == #eof ifTrue: [^LispCons null].
	tokenType == #rightParenthesis ifTrue: [^LispCons null].
	tokenType == #leftParenthesis ifTrue: [^LispCons head: self scanListAux tail: self scanListAux].
	tokenType == #number ifTrue: [^LispCons head: token tail: self scanListAux].
	tokenType == #string ifTrue: [^LispCons head: token tail: self scanListAux].
	tokenType == #object ifTrue: [^LispCons head: token tail: self scanListAux].
	tokenType == #symbol ifTrue: [^LispCons head: token tail: self scanListAux].
	tokenType == #period
		ifTrue: 
			[cdr := self scanList.
			self nextToken.
			tokenType == #rightParenthesis
				ifTrue: [^cdr]
				ifFalse: [^failBlock value: 'Syntax error']].
	tokenType == #quote
		ifTrue: 
			[cdr := LispCons head: self scanList tail: LispCons null.
			cdr := LispCons head: #quote tail: cdr.
			^LispCons head: cdr tail: self scanListAux].
	self unNextToken.
	^failBlock value: 'Syntax error'! !


!LispParser methodsFor: 'private'!
init: sourceStream ifFail: aBlock 
	super on: sourceStream.
	failBlock := aBlock! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

LispParser class
	instanceVariableNames: ''!

!LispParser class methodsFor: 'copyright'!
copyright
	^'Copyright (C) 1995-1998 AOKI Atsushi, All Rights Reserved.'! !

!LispParser class methodsFor: 'copyright'!
system
	^'Goodies'! !

!LispParser class methodsFor: 'copyright'!
version
	^'003'! !


!LispParser class methodsFor: 'utilities'!
parse: aStream 
	^self new parse: (self makeStream: aStream)! !

!LispParser class methodsFor: 'utilities'!
parse: aStream ifFail: aBlock 
	^self new parse: (self makeStream: aStream)
		ifFail: aBlock! !


!LispParser class methodsFor: 'private'!
makeStream: aStream 
	| stream |
	((aStream isKindOf: String)
		or: [aStream isKindOf: Text])
		ifTrue: [stream := ReadStream on: aStream asString]
		ifFalse: [stream := aStream].
	^stream! !


!LispParser class methodsFor: 'examples'!
example1
	"LispParser example1."

	| list |
	list := LispParser parse: '(1 2 3 4 (5 6 7 8 9) 10)'.
	^list! !

!LispParser class methodsFor: 'examples'!
example2
	"LispParser example2."

	| list |
	list := LispParser parse: '
			(10 (1 2)
			 20 (3 4 . 100)
			 30 (5 6) . 200)
	'.
	^list! !

!LispParser class methodsFor: 'examples'!
example3
	"LispParser example3."

	| list |
	list := LispParser parse: '
			(PetriNet Aoki
				(Place
					(p1 . {100@100})
					(p2 . {200@200})
					(p3 . {300@300})
					(p4 . {400@400})
					(p5 . {500@500}))
				(Transition
					(t1 . {100@100})
					(t2 . {200@200})
					(t3 . {300@300})
					(t4 . {400@400})
					(t5 . {500@500}))
				(InputFunction
					(t1 p1 p2 p3 p4 p5)
					(t2 . p4)
					(t3 . p5))
				(OutputFunction
					(t1 p1 p2 p3 p4 p5)
					(t2 . p4)
					(t3 . p5))
				(Marking {#(1 2 3 4 5)}))'.
	Transcript cr; show: list saveString.
	list := LispParser parse: list saveString.
	^list! !

!LispParser class methodsFor: 'examples'!
example4
	"LispParser example4."

	| list |
	list := LispParser parse: '(1 2 3 4 (5 6 7 ~ 8 9) 10)'.
	^list! !

!LispParser class methodsFor: 'examples'!
example5
	"LispParser example5."

	| list |
	list := LispParser parse: '(1 2 3 4 (5 6 7 {100@100 8 9) 10)'.	"error"
	^list! !

!LispParser class methodsFor: 'examples'!
example6
	"LispParser example6."

	| list |
	list := LispParser parse: '(1 2 3 4 (5 6 7 ''aaaaa 8 9) 10)'.	"error"
	^list! !

!LispParser class methodsFor: 'examples'!
example7
	"LispParser example7."

	| list |
	list := LispParser parse: ' `(`(1 2 `3) . `4) '.	"`(`(1 2 `3) . `4) -> (quote ((quote (1 2 (quote 3))) quote 4))"
	^list! !
